<?php
/**
 * Template class
 *
 * @author PS
 * @Package WPPCT
 * @since 1.0.0
 * @version 1.2.5
 */

namespace InnoMobile;
use \WP_Query;

class WP_Post_Template {
    var $wppct_post_type;
    var $admin_url;
    var $capability_use = 'wppct_use_templates';
    var $capability_create = 'wppct_create_templates';

    public function __construct($post_type_slug) {
        $this->wppct_post_type = $post_type_slug;
        $this->admin_url = get_admin_url();
        add_action('init', array($this, 'init'), 10 );
        add_action('admin_init', array($this, 'admin_init'), 10 );
        
        add_action( 'admin_action_wppct_duplicate_post', array($this, 'wppct_duplicate_post') );
        add_action( 'admin_action_wppct_templatize_post', array($this, 'wppct_templatize_post') );
        add_action( 'admin_action_wppct_use_template', array($this, 'wppct_use_template') );
        add_filter( 'manage_posts_columns', array($this, 'wppct_add_column'), 10, 2 );
        add_action( 'manage_posts_custom_column', array($this, 'wppct_column_template_name'), 10, 2 );
        add_filter( 'manage_edit-'.$this->wppct_post_type.'_sortable_columns', array($this, 'wppct_manage_sortable_columns'));
        add_action( 'pre_get_posts', array($this, 'manage_wp_posts_pre_get_posts'), 1 );
    }
    
    public function get_post_type_name() {
        return $this->wppct_post_type;
    }

    public function init() {		
        $menu_icon = null;

        register_post_type(
            $this->wppct_post_type,
            array(
                'label' => 'Templates',
                'description' => '',
                'public' => true,
                'show_ui' => true,
                'show_in_menu' => true,
				'publicly_queryable' => false,
				'show_in_nav_menus' => false,
                'capability_type' => 'post',
                'menu_icon' => $menu_icon, 
                'hierarchical' => false,
                'rewrite' => array('slug' => ''),
                'query_var' => true,
                'exclude_from_search' => true,
                'supports' => array('title','editor','excerpt','custom-fields','comments','revisions','thumbnail','author','page-attributes','post-formats'),
                'taxonomies' => array(),
                'labels' => array (
                  'name' => __('Templates', 'wppct'),
                  'singular_name' => __('Template', 'wppct'),
                  'menu_name' => __('Templates', 'wppct'),
                  'add_new' => __('Add Template', 'wppct'),
                  'add_new_item' => __('Add New Template', 'wppct'),
                  'edit' => __('Edit', 'wppct'),
                  'edit_item' => __('Edit Template', 'wppct'),
                  'new_item' => __('New Template', 'wppct'),
                  'view' => __('View Template', 'wppct'),
                  'view_item' => __('View Template', 'wppct'),
                  'search_items' => __('Search Templates', 'wppct'),
                  'not_found' => __('No Templates Found', 'wppct'),
                  'not_found_in_trash' => __('No Templates Found in Trash', 'wppct'),
                  'parent' => __('Parent Template', 'wppct'),
                )
            ) 
        );

        //CPT get all available taxonomies, so you can use everyone for your template
        $this->add_taxonomies();

        //Add Links to Backend
        $this->add_links();   
    }
    
    public function admin_init() {
        if(class_exists('\InnoMobile\Dashboard_Widget')) {
            $code = $this->generate_widget_code();
            $WPPCT_dashboard = new Dashboard_Widget('wppct_template_dashboard_widget', __('Templates', 'wppct'), $code, $this->capability_use);
        }
    }

    public function wppct_duplicate_post() {
        if(!isset($_REQUEST['_wppct_nonce']) || !wp_verify_nonce( $_REQUEST['_wppct_nonce'], 'wppct_nonce' )) {
           print 'Sorry, your nonce did not verify.';
           exit;
        }
        $this->wppct_new_post('duplicate');
    }

    public function wppct_templatize_post() {
        if(!isset($_REQUEST['_wppct_nonce']) || !wp_verify_nonce( $_REQUEST['_wppct_nonce'], 'wppct_nonce' )) {
           print 'Sorry, your nonce did not verify.';
           exit;
        }
        $this->wppct_new_post('templatize');
    }   
    
    public function wppct_use_template() {
        if(!isset($_REQUEST['_wppct_nonce']) || !wp_verify_nonce( $_REQUEST['_wppct_nonce'], 'wppct_nonce' )) {
           print 'Sorry, your nonce did not verify.';
           exit;
        }
        $this->wppct_new_post('use');
    }
    
    public function wppct_new_post($variante, $template_id = -1, $copy_id = -1) {
            if(!current_user_can('wppct_use_templates')) {
                print 'Sorry, you have not the rights to use templates!';
                exit;
            } else {                
                //Erhalte die ID des Posts
                if($template_id==-1) $template_id = $_REQUEST['post'];
                
                switch($variante) {
                    case 'use' :
                        $template_post_type = $_REQUEST['post_type'];
                        
                        $count_placeholders = count(get_post_meta($template_id, '_'.$this->get_post_type_name().'_placeholders',true));
                        
                        $template_placeholder = array();
                        for($i=0;$i<$count_placeholders;$i++) {
                            $template_placeholder[$i] = $_REQUEST['template_placeholder_'.$i];
                        }
                        break;
                    case 'templatize' :
                        $template_post_type = $this->wppct_post_type;
                        $template_placeholder = '';
                        break;
                    case 'custom':
                        $template_post_type = $_REQUEST['post_type'];
                        $template_placeholder = '';
                        break;
                    default :
                        $template_post_type = '';
                        $template_placeholder = '';
                        break;
                }

                //Schritt 1: Extrahiere die Daten dieses Posts
                $post_data = $this->extract_data($template_id);

                // Schaue zunächst, ob die Post ID existiert
                if(count($post_data)==0) {
                    // Post ID existiert nicht
                    wp_redirect( $this->admin_url.'?error=notemplate' );
                    exit;
                }

                //Überschreibe den Post Type, wenn das Template verwendet werden soll oder ein Template erstellt wird
                if(!empty($template_post_type)) {
                    $post_data['post_type'] = $template_post_type;                
                }
                if($variante=='use' && $post_data['post_type']==$this->wppct_post_type) $post_data['post_type'] = 'post';

                //Ist ein Wert für den Platzhalter eingetragen, dann tausche diesen aus
                if(is_array($template_placeholder) && !empty($template_placeholder)) {
                    $post_data = $this->replace_placeholder($template_id, $post_data, $template_placeholder);
                }
                
                //Filter wird genutzt bei der Variante custom
                if($variante=='custom') {
                    $post_data = apply_filters('wppct_replace', $post_data, $copy_id );
                }

                //Erstelle nun den neuen Post
                $post_id = $this->create_post($post_data);

                //Füge Taxonimien hinzu
                $this->update_taxonomies($template_id, $post_id);
                //Füge benutzerdefinierte Felder hinzu
                $this->update_custom_fields($template_id, $post_id);
                //Lege die Anhaenge fest
                $this->update_attachments($template_id, $post_id);
                
                // Do something if custom variante
                if($variante=='custom') {
                    do_action('wppct_update_post', $post_id, $copy_id );
                }                
                
                
                //Prüfe, ob Post erstellt werden konnte
                if($post_id > 0) {
                    wp_redirect( get_edit_post_link($post_id, '') );
                    exit;
                } else {
                    wp_redirect( $this->admin_url.'?error=notcreated' );
                    exit;
                }   
            }
        
    }
    
    private function replace_placeholder($template_id, $post_data, $replace_placeholder) {
        //Finde zunächst heraus, ob es einen Platzhalter im Template gibt
        $placeholder = get_post_meta($template_id, '_'.$this->get_post_type_name().'_placeholders',true);
        
        for($i=0;$i<count($placeholder);$i++) {
            $to_replace = $placeholder[$i]["_wppct_placeholder"];
            $replace_with = $replace_placeholder[$i];
            if(!empty($replace_with)) {
                $post_data['post_title'] = str_replace($to_replace, $replace_with, $post_data['post_title']);
                $post_data['post_content'] = str_replace($to_replace, $replace_with, $post_data['post_content']);
                $post_data['post_excerpt'] = str_replace($to_replace, $replace_with, $post_data['post_excerpt']);
            }
        }        
        return $post_data;
    }
    

    
    private function extract_data($post_id) {
        $new_post = array();
        $old_post = get_post($post_id);

        if(!empty($old_post)) {
            $properties = get_object_vars($old_post);
            $irrelevant_properties = array(
                'ID',
                'post_author',
                'post_modified',
                'post_modified_gmt',
                'post_name',
                'guid',
                'post_status',
                'post_date',
                'post_date_gmt',
            );

            $new_post = array();
            $new_post['post_status'] = 'draft';

            foreach($properties as $property => $value) {
                if( !in_array($property, $irrelevant_properties) ) {
                    $new_post[$property] = $value;
                }
            }
        }
        return $new_post;
    }
    
    private function create_post($post_data) {
        $user_id = get_current_user_id();
        $post_data['post_author'] = $user_id;
        
        $new_post_id = wp_insert_post($post_data);
        return $new_post_id;
    }
    
    /**
     * Update taxonomies of new article
     * @param type $old_post_id Post where you want to copy the data
     * @param type $new_post_id New post where the data from old post should be insert
     */
    private function update_taxonomies($old_post_id, $new_post_id) {
        $taxonomies = get_taxonomies();

        foreach($taxonomies as $taxonomy) {
            $template_terms = wp_get_object_terms($old_post_id, $taxonomy);
            $add = array();

            foreach($template_terms as $term) {
                $add[] = $term->slug;
            }

            wp_set_object_terms($new_post_id, $add, $taxonomy);
        }
    }
    
    /**
     * Insert attachments to new article
     * @param type $old_post_id Post where you want to copy the data
     * @param type $new_post_id New post where the data from old post should be insert
     */
    private function update_attachments($old_post_id, $new_post_id) {
        $args = array(
            'post_type' => 'attachment',
            'numberposts' => null,
            'post_status' => null,
            'post_parent' => $old_post_id
        );
        $attachments = get_posts($args);

        foreach($attachments as $attachment) {
            $image = wp_get_attachment_metadata($attachment->ID);
            wp_insert_attachment($image, false, $new_post_id);
        }
    }
    
    /**
     * Update custom fields of new article
     * @param type $old_post_id Post where you want to copy the data
     * @param type $new_post_id New post where the data from old post should be insert
     */
    private function update_custom_fields($old_post_id, $new_post_id) {
        $custom_fields = get_post_custom_keys($old_post_id);

        foreach($custom_fields as $custom_field) {
            $value = get_post_meta($old_post_id, $custom_field, true);
            update_post_meta($new_post_id, $custom_field, $value);
        } 
    }    
    private function add_links() {
        // for post and custom post type
        add_filter('post_row_actions', array($this, 'add_wppct_post_link'), 10, 2);
        // for page
        add_filter('page_row_actions', array($this, 'add_wppct_post_link'), 10, 2);
    }

    /**
     * 
     * @param string $actions WP specific
     * @param type $post WP Post
     * @return string $actions
     */
    public function add_wppct_post_link($actions, $post) {
        $posttype = $this->get_post_types('array');
        $nonce = wp_create_nonce( 'wppct_nonce' );

        if($post->post_type == $this->wppct_post_type) {
            $actions['duplicate'] = $this->get_link_url($post->ID, 'wppct_duplicate_post', __('Duplicate', 'wppct'));
            $actions['templatize'] = $this->get_link_url($post->ID, 'wppct_use_template', __('Create post using this template', 'wppct'));
        } elseif(in_array($post->post_type, $posttype)) {
            $actions['duplicate'] = $this->get_link_url($post->ID, 'wppct_duplicate_post', __('Duplicate', 'wppct'));
            $actions['templatize'] = $this->get_link_url($post->ID, 'wppct_templatize_post', __('Templatize', 'wppct'));
        }

        return $actions; 
    }      
    
    private function get_link_url($post_id, $name, $text) {
        $nonce = wp_nonce_url('admin.php?action='.$name.'&amp;post=' . $post_id, 'wppct_nonce', '_wppct_nonce');
        $link = '<a href="'.$nonce.'" title="'.$text.'" rel="permalink">'.$text.'</a>';
        return $link;
        
    }

    public function get_templates() {
        //Query all template IDs and Title
        $post_templates = array();
        $post_type = $this->get_post_type_name();
        $query = new WP_Query( array( 'post_type' => $post_type, 'post_status' => 'publish', 'order' => 'ASC', 'orderby' => 'title', 'posts_per_page'=>-1 ) );
        if ( $query->have_posts() ) {
            while ( $query->have_posts() ) {
                    $query->the_post();
                    
                    // Trage Namen für Template ein
                    $template_title = $this->get_value(get_the_ID(), '_'.$this->get_post_type_name().'_template_name', get_the_title());
                    
                    //Trage, falls vorhanden, den Platzhalter ein
                    $template_placeholder = get_post_meta(get_the_ID(), '_'.$this->get_post_type_name().'_placeholders',true);
                    //if(get_the_ID()==26047) die(var_dump($template_placeholder[0]["_wppct_placeholder"]));
                    
                    //Trage, falls vorhanden, eine Kategorie ein
                    $template_category = $this->get_value(get_the_ID(), '_'.$this->get_post_type_name().'_category', '');
                    
                    $post_templates[] = array(
                        'ID' => get_the_ID(),
                        'title' => $template_title,
                        'placeholder' => $template_placeholder,
                        'category' => $template_category,
                    );
            }
            $this->array_sort_by_column($post_templates, 'title');
        }
        /* Restore original Post Data */
        wp_reset_postdata();            

        return $post_templates;
    }

    public function has_templates() {
        $post_type = $this->get_post_type_name();
        $query = new WP_Query( array( 'post_type' => $post_type ) );
        if ( $query->have_posts() ) {
            return true;
        }
        return false;
    } 
    
    public function get_value($post_id, $key, $default='', $array=false) {
        $meta = get_post_meta($post_id);
        
        if(isset($meta[$key])) {
            if(!empty($meta[$key][0])) {
                $value = $meta[$key][0];
                return $value;
            }
        }
        
        $value = $default;
        return $value;
    }

    public function get_post_types($variante) {
        if(empty($variante)) $variante = 'object';
        $post_type_objects = array();
        $post_types = get_post_types( array('public' => true, 'show_ui' => true) );
        unset($post_types['attachment']);

        if($variante == "array") {
            foreach ($post_types as $post_type)
            {
                $post_type_objects[] = $post_type;
            }                
        } else {
            foreach($post_types as $post_type) {
                    $post_type_objects[] = get_post_type_object($post_type);
            }
        }
        return $post_type_objects;
    }

    /**
     * Add all Taxonomies to CPT
     */
    private function add_taxonomies() {
        $args = array(
           'public'   => true
        );
        $taxonomies = get_taxonomies( $args );
        foreach ( $taxonomies  as $taxonomy ) {
           register_taxonomy_for_object_type( $taxonomy, $this->wppct_post_type );
        }
    }
    
    private function array_sort_by_column(&$arr, $col, $dir = SORT_ASC, $sort_flags = SORT_NATURAL ) {
        $sort_col = array();
        foreach ($arr as $key=> $row) {
            $sort_col[$key] = $row[$col];
        }
        array_multisort($sort_col, $dir, $sort_flags, $arr);
    }

    
    /**
     * Log messages
     * @param string $message Which message should appear in log-file
     */
    private function log($message) {
        if (WP_DEBUG === true) {
                if (is_array($message) || is_object($message)) {
                        error_log(print_r($message, true));
                } else {
                        error_log($message);
                }
        }
    }
    
    
    /**
     * Dashboard Widget
     */
    
    function generate_widget_code() {
        $code = '';
        $p0 = __('You have no templates!', 'wppct');
        $p1 = __('Choose your template and click on create!', 'wppct');
        $p2 = __('Use your posts to templatize a new one', 'wppct');
        $p3 = __('or', 'wppct');
        $url = $this->admin_url.'edit.php?post_type='.$this->get_post_type_name();
        $p4 = __('create a template from scratch', 'wppct');
        
        $string_template = __('Template','wppct');
        $string_post_type = __('Create post in','wppct');
        $string_create = __('Create','wppct');
       
        if($this->has_templates()) {           
            $code .= sprintf('<p>%1$s %2$s %3$s <a href="%4$s">%5$s</a>!</p>', $p1, $p2, $p3, $url, $p4);
            $code .= '<form id="create-page-from-template" action="'.$this->admin_url.'admin.php" method="get">';
                $code .= '<input type="hidden" name="action" value="wppct_use_template" />';

                        $code .= '<p><strong>'.$string_template.':</strong><select onChange="wppct_placeholder()" id="select_template_id" name="post" style="width:100%">';
                            $post_templates = $this->get_templates();                       
                            foreach($post_templates as $post_template) {
                                $class_placeholder = '';
                                if(!empty($post_template['placeholder'])) {
                                    
                                    for($i=0;$i<count($post_template['placeholder']); $i++) {
                                        $placeholder = $post_template['placeholder'][$i]["_wppct_placeholder"];
                                        $class_placeholder .= 'data-placeholder_'.$i.'="'.$placeholder.'" ';
                                                
                                    }
                                    
                                    
                                } 
                                if(!empty($post_template['category'])) {
                                    $class_category = 'category="'.$post_template['category'].'"';
                                } else {
                                    $class_category = 'category=""';
                                }                                
                                $code .= '<option '.$class_placeholder.' '.$class_category.' value="'.$post_template['ID'].'">';
                                $code .= $post_template['title'];      
                                $code .= '</option>';
                            }
                        $code .= '</select></p>';
                        $placeholder_title = __('Replace','wppct').' <span>[DATA_VALUE]</span> '.__('with','wppct');
                        $code .= '<div class="js_template_placeholder" style="display:none;">'
                                  . '<div class="my_template_placeholder">'
                                   . '<strong>'.$placeholder_title.':</strong>'
                                   . '<input style="width:100%" type="text" name="template_[DATA_NAME]" value="" />'
                                . '</div>'
                               . '</div>';
                        
                        $code .= '<div class="add_template_placeholder"><strong>'.__('Placeholder','wppct').'</strong>'
                                . '<div id="template_placeholder_empty"></div>'
                                . '<br />'.__('This template contains a placeholder. What should it replaced with?', 'wppct').''
                                . '</div>';
                        
                        // List Post Types
                        $code .= '<p><strong>'.$string_post_type.':</strong><select name="post_type" style="width:100%">';
                            $post_types = $this->get_post_types('object');
                            foreach($post_types as $post_type) {
                                //print_r($post_type);
                                $code .= '<option value="'.$post_type->name.'">';
                                $code .= $post_type->labels->singular_name;        
                                $code .= '</option>';
                            }
                        $code .= '</select></p>';
                        // Nonce Field
                        $nonce = wp_create_nonce( 'wppct_nonce' );
                        $code .= '<input type="hidden" id="_wppct_nonce" name="_wppct_nonce" value="'.$nonce.'" />';
                        $code .= '<input type="submit" name="submit" class="button-secondary" value="'.$string_create.'" />';

            $code .= '</form>';
        } else {
            $code = sprintf('<p>%1$s %2$s %3$s <a href="%4$s">%5$s</a>!</p>', $p0, $p2, $p3, $url, $p4);
        }        
        return $code;
    }   
    
    public function wppct_add_column( $columns, $post_type ) {
        if ($post_type == $this->wppct_post_type) {
            $new_columns = array();
            foreach ($columns as $key => $value) {
                
                if ($key == 'title')
                    $new_columns['template_name'] = 'Template Name';
                $new_columns[$key] = $value;
            }
            return $new_columns;
        }
        return $columns;
    }
    
    
    function wppct_column_template_name( $column_name, $post_id ) {
       switch( $column_name ) {
          case 'template_name':
             $template_title = $this->get_value(get_the_ID(), '_'.$this->get_post_type_name().'_template_name', '');
             echo '<div id="template_name-' . $post_id . '"><a href="'.get_edit_post_link( $post_id ).'">' . $template_title . '</a></div>';
             break;
       }
    }
    
    function wppct_manage_sortable_columns( $sortable_columns ) {
       $sortable_columns[ 'template_name' ] = 'template_name_order';
       return $sortable_columns;
    }
    
    function manage_wp_posts_pre_get_posts( $query ) {
       if(isset($_GET['orderby'])) {
            if ( $query->is_main_query() && ( $_GET['orderby'] == 'template_name_order' ) ) {
                    $template_name = '_'.$this->get_post_type_name().'_template_name';
                    $query->set( 'meta_key', $template_name );
                    $query->set( 'orderby', 'meta_value' );	
                    $query->set( 'order', $_GET['order'] );	
                    
            }
       }
    }
}